# SPDX-License-Identifier: BSD-2-Clause

# Copyright (c) 2024 Phil Thompson <phil@riverbankcomputing.com>


from .argument_properties_dialog import ArgumentPropertiesDialog
from .callable_properties_dialog import CallablePropertiesDialog
from .class_properties_dialog import ClassPropertiesDialog
from .enum_properties_dialog import EnumPropertiesDialog
from .enum_member_properties_dialog import EnumMemberPropertiesDialog
from .features_dialog import FeaturesDialog
from .manual_code_dialog import ManualCodeDialog
from .module_properties_dialog import ModulePropertiesDialog
from .move_header_dialog import MoveHeaderDialog
from .namespace_properties_dialog import NamespacePropertiesDialog
from .opaque_class_properties_dialog import OpaqueClassPropertiesDialog
from .platforms_dialog import PlatformsDialog
from .project_properties_dialog import ProjectPropertiesDialog
from .typedef_properties_dialog import TypedefPropertiesDialog
from .variable_properties_dialog import VariablePropertiesDialog
from .versions_dialog import VersionsDialog
