# SPDX-License-Identifier: BSD-2-Clause

# Copyright (c) 2024 Phil Thompson <phil@riverbankcomputing.com>


from .header_directory import (get_platform_name, get_supported_platforms,
        header_directory_platform)
from .version_map import VersionMap
from .version_range import version_range
